# coding=utf-8
# pylint: disable=too-many-lines
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import datetime
from typing import Any, Dict, List, Optional, TYPE_CHECKING, Union

from .. import _serialization

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from .. import models as _models


class Error(_serialization.Model):
    """Azure App Configuration error object.

    :ivar type: The type of the error.
    :vartype type: str
    :ivar title: A brief summary of the error.
    :vartype title: str
    :ivar name: The name of the parameter that resulted in the error.
    :vartype name: str
    :ivar detail: A detailed description of the error.
    :vartype detail: str
    :ivar status: The HTTP status code that the error maps to.
    :vartype status: int
    """

    _attribute_map = {
        "type": {"key": "type", "type": "str"},
        "title": {"key": "title", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "detail": {"key": "detail", "type": "str"},
        "status": {"key": "status", "type": "int"},
    }

    def __init__(
        self,
        *,
        type: Optional[str] = None,
        title: Optional[str] = None,
        name: Optional[str] = None,
        detail: Optional[str] = None,
        status: Optional[int] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword type: The type of the error.
        :paramtype type: str
        :keyword title: A brief summary of the error.
        :paramtype title: str
        :keyword name: The name of the parameter that resulted in the error.
        :paramtype name: str
        :keyword detail: A detailed description of the error.
        :paramtype detail: str
        :keyword status: The HTTP status code that the error maps to.
        :paramtype status: int
        """
        super().__init__(**kwargs)
        self.type = type
        self.title = title
        self.name = name
        self.detail = detail
        self.status = status


class ErrorDetail(_serialization.Model):
    """The details of an error.

    All required parameters must be populated in order to send to Azure.

    :ivar code: One of a server-defined set of error codes. Required.
    :vartype code: str
    :ivar message: A human-readable representation of the error. Required.
    :vartype message: str
    :ivar details: An array of details about specific errors that led to this reported error.
    :vartype details: list[~azure.appconfiguration.models.ErrorDetail]
    :ivar innererror: An object containing more specific information than the current object about
     the error.
    :vartype innererror: ~azure.appconfiguration.models.InnerError
    """

    _validation = {
        "code": {"required": True},
        "message": {"required": True},
    }

    _attribute_map = {
        "code": {"key": "code", "type": "str"},
        "message": {"key": "message", "type": "str"},
        "details": {"key": "details", "type": "[ErrorDetail]"},
        "innererror": {"key": "innererror", "type": "InnerError"},
    }

    def __init__(
        self,
        *,
        code: str,
        message: str,
        details: Optional[List["_models.ErrorDetail"]] = None,
        innererror: Optional["_models.InnerError"] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword code: One of a server-defined set of error codes. Required.
        :paramtype code: str
        :keyword message: A human-readable representation of the error. Required.
        :paramtype message: str
        :keyword details: An array of details about specific errors that led to this reported error.
        :paramtype details: list[~azure.appconfiguration.models.ErrorDetail]
        :keyword innererror: An object containing more specific information than the current object
         about the error.
        :paramtype innererror: ~azure.appconfiguration.models.InnerError
        """
        super().__init__(**kwargs)
        self.code = code
        self.message = message
        self.details = details
        self.innererror = innererror


class InnerError(_serialization.Model):
    """An object containing specific information about an error.

    :ivar code: One of a server-defined set of error codes.
    :vartype code: str
    :ivar innererror: An object containing more specific information than the current object about
     the error.
    :vartype innererror: ~azure.appconfiguration.models.InnerError
    """

    _attribute_map = {
        "code": {"key": "code", "type": "str"},
        "innererror": {"key": "innererror", "type": "InnerError"},
    }

    def __init__(
        self, *, code: Optional[str] = None, innererror: Optional["_models.InnerError"] = None, **kwargs: Any
    ) -> None:
        """
        :keyword code: One of a server-defined set of error codes.
        :paramtype code: str
        :keyword innererror: An object containing more specific information than the current object
         about the error.
        :paramtype innererror: ~azure.appconfiguration.models.InnerError
        """
        super().__init__(**kwargs)
        self.code = code
        self.innererror = innererror


class Key(_serialization.Model):
    """Key.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: The name of the key.
    :vartype name: str
    """

    _validation = {
        "name": {"readonly": True},
    }

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.name = None


class KeyListResult(_serialization.Model):
    """The result of a list request.

    :ivar items: The collection value.
    :vartype items: list[~azure.appconfiguration.models.Key]
    :ivar next_link: The URI that can be used to request the next set of paged results.
    :vartype next_link: str
    """

    _attribute_map = {
        "items": {"key": "items", "type": "[Key]"},
        "next_link": {"key": "@nextLink", "type": "str"},
    }

    def __init__(
        self, *, items: Optional[List["_models.Key"]] = None, next_link: Optional[str] = None, **kwargs: Any
    ) -> None:
        """
        :keyword items: The collection value.
        :paramtype items: list[~azure.appconfiguration.models.Key]
        :keyword next_link: The URI that can be used to request the next set of paged results.
        :paramtype next_link: str
        """
        super().__init__(**kwargs)
        self.items = items
        self.next_link = next_link


class KeyValue(_serialization.Model):
    """KeyValue.

    :ivar key: The key of the key-value.
    :vartype key: str
    :ivar label: The label the key-value belongs to.
    :vartype label: str
    :ivar content_type: The content type of the value stored within the key-value.
    :vartype content_type: str
    :ivar value: The value of the key-value.
    :vartype value: str
    :ivar last_modified: A date representing the last time the key-value was modified.
    :vartype last_modified: ~datetime.datetime
    :ivar tags: The tags of the key-value.
    :vartype tags: dict[str, str]
    :ivar locked: Indicates whether the key-value is locked.
    :vartype locked: bool
    :ivar etag: A value representing the current state of the resource.
    :vartype etag: str
    """

    _attribute_map = {
        "key": {"key": "key", "type": "str"},
        "label": {"key": "label", "type": "str"},
        "content_type": {"key": "content_type", "type": "str"},
        "value": {"key": "value", "type": "str"},
        "last_modified": {"key": "last_modified", "type": "iso-8601"},
        "tags": {"key": "tags", "type": "{str}"},
        "locked": {"key": "locked", "type": "bool"},
        "etag": {"key": "etag", "type": "str"},
    }

    def __init__(
        self,
        *,
        key: Optional[str] = None,
        label: Optional[str] = None,
        content_type: Optional[str] = None,
        value: Optional[str] = None,
        last_modified: Optional[datetime.datetime] = None,
        tags: Optional[Dict[str, str]] = None,
        locked: Optional[bool] = None,
        etag: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword key: The key of the key-value.
        :paramtype key: str
        :keyword label: The label the key-value belongs to.
        :paramtype label: str
        :keyword content_type: The content type of the value stored within the key-value.
        :paramtype content_type: str
        :keyword value: The value of the key-value.
        :paramtype value: str
        :keyword last_modified: A date representing the last time the key-value was modified.
        :paramtype last_modified: ~datetime.datetime
        :keyword tags: The tags of the key-value.
        :paramtype tags: dict[str, str]
        :keyword locked: Indicates whether the key-value is locked.
        :paramtype locked: bool
        :keyword etag: A value representing the current state of the resource.
        :paramtype etag: str
        """
        super().__init__(**kwargs)
        self.key = key
        self.label = label
        self.content_type = content_type
        self.value = value
        self.last_modified = last_modified
        self.tags = tags
        self.locked = locked
        self.etag = etag


class KeyValueFilter(_serialization.Model):
    """Enables filtering of key-values.

    All required parameters must be populated in order to send to Azure.

    :ivar key: Filters key-values by their key field. Required.
    :vartype key: str
    :ivar label: Filters key-values by their label field.
    :vartype label: str
    """

    _validation = {
        "key": {"required": True},
    }

    _attribute_map = {
        "key": {"key": "key", "type": "str"},
        "label": {"key": "label", "type": "str"},
    }

    def __init__(self, *, key: str, label: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword key: Filters key-values by their key field. Required.
        :paramtype key: str
        :keyword label: Filters key-values by their label field.
        :paramtype label: str
        """
        super().__init__(**kwargs)
        self.key = key
        self.label = label


class KeyValueListResult(_serialization.Model):
    """The result of a list request.

    :ivar items: The collection value.
    :vartype items: list[~azure.appconfiguration.models.KeyValue]
    :ivar etag: An identifier representing the returned state of the resource.
    :vartype etag: str
    :ivar next_link: The URI that can be used to request the next set of paged results.
    :vartype next_link: str
    """

    _attribute_map = {
        "items": {"key": "items", "type": "[KeyValue]"},
        "etag": {"key": "etag", "type": "str"},
        "next_link": {"key": "@nextLink", "type": "str"},
    }

    def __init__(
        self,
        *,
        items: Optional[List["_models.KeyValue"]] = None,
        etag: Optional[str] = None,
        next_link: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword items: The collection value.
        :paramtype items: list[~azure.appconfiguration.models.KeyValue]
        :keyword etag: An identifier representing the returned state of the resource.
        :paramtype etag: str
        :keyword next_link: The URI that can be used to request the next set of paged results.
        :paramtype next_link: str
        """
        super().__init__(**kwargs)
        self.items = items
        self.etag = etag
        self.next_link = next_link


class Label(_serialization.Model):
    """Label.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar name: The name of the label.
    :vartype name: str
    """

    _validation = {
        "name": {"readonly": True},
    }

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
    }

    def __init__(self, **kwargs: Any) -> None:
        """ """
        super().__init__(**kwargs)
        self.name = None


class LabelListResult(_serialization.Model):
    """The result of a list request.

    :ivar items: The collection value.
    :vartype items: list[~azure.appconfiguration.models.Label]
    :ivar next_link: The URI that can be used to request the next set of paged results.
    :vartype next_link: str
    """

    _attribute_map = {
        "items": {"key": "items", "type": "[Label]"},
        "next_link": {"key": "@nextLink", "type": "str"},
    }

    def __init__(
        self, *, items: Optional[List["_models.Label"]] = None, next_link: Optional[str] = None, **kwargs: Any
    ) -> None:
        """
        :keyword items: The collection value.
        :paramtype items: list[~azure.appconfiguration.models.Label]
        :keyword next_link: The URI that can be used to request the next set of paged results.
        :paramtype next_link: str
        """
        super().__init__(**kwargs)
        self.items = items
        self.next_link = next_link


class OperationDetails(_serialization.Model):
    """Details of a long running operation.

    All required parameters must be populated in order to send to Azure.

    :ivar id: The unique id of the operation. Required.
    :vartype id: str
    :ivar status: The current status of the operation. Required. Known values are: "NotStarted",
     "Running", "Succeeded", "Failed", and "Canceled".
    :vartype status: str or ~azure.appconfiguration.models.State
    :ivar error: An error, available when the status is ``Failed``\ , describing why the operation
     failed.
    :vartype error: ~azure.appconfiguration.models.ErrorDetail
    """

    _validation = {
        "id": {"required": True},
        "status": {"required": True},
    }

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "status": {"key": "status", "type": "str"},
        "error": {"key": "error", "type": "ErrorDetail"},
    }

    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        status: Union[str, "_models.State"],
        error: Optional["_models.ErrorDetail"] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword id: The unique id of the operation. Required.
        :paramtype id: str
        :keyword status: The current status of the operation. Required. Known values are: "NotStarted",
         "Running", "Succeeded", "Failed", and "Canceled".
        :paramtype status: str or ~azure.appconfiguration.models.State
        :keyword error: An error, available when the status is ``Failed``\ , describing why the
         operation failed.
        :paramtype error: ~azure.appconfiguration.models.ErrorDetail
        """
        super().__init__(**kwargs)
        self.id = id
        self.status = status
        self.error = error


class Snapshot(_serialization.Model):  # pylint: disable=too-many-instance-attributes
    """Snapshot.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar name: The name of the snapshot.
    :vartype name: str
    :ivar status: The current status of the snapshot. Known values are: "provisioning", "ready",
     "archived", and "failed".
    :vartype status: str or ~azure.appconfiguration.models.SnapshotStatus
    :ivar filters: A list of filters used to filter the key-values included in the snapshot.
     Required.
    :vartype filters: list[~azure.appconfiguration.models.KeyValueFilter]
    :ivar composition_type: The composition type describes how the key-values within the snapshot
     are composed. The 'key' composition type ensures there are no two key-values containing the
     same key. The 'key_label' composition type ensures there are no two key-values containing the
     same key and label. Known values are: "key" and "key_label".
    :vartype composition_type: str or ~azure.appconfiguration.models.CompositionType
    :ivar created: The time that the snapshot was created.
    :vartype created: ~datetime.datetime
    :ivar expires: The time that the snapshot will expire.
    :vartype expires: ~datetime.datetime
    :ivar retention_period: The amount of time, in seconds, that a snapshot will remain in the
     archived state before expiring. This property is only writable during the creation of a
     snapshot. If not specified, the default lifetime of key-value revisions will be used.
    :vartype retention_period: int
    :ivar size: The size in bytes of the snapshot.
    :vartype size: int
    :ivar items_count: The amount of key-values in the snapshot.
    :vartype items_count: int
    :ivar tags: The tags of the snapshot.
    :vartype tags: dict[str, str]
    :ivar etag: A value representing the current state of the snapshot.
    :vartype etag: str
    """

    _validation = {
        "name": {"readonly": True},
        "status": {"readonly": True},
        "filters": {"required": True, "max_items": 3, "min_items": 1},
        "created": {"readonly": True},
        "expires": {"readonly": True},
        "retention_period": {"maximum": 7776000, "minimum": 3600},
        "size": {"readonly": True},
        "items_count": {"readonly": True},
        "etag": {"readonly": True},
    }

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
        "status": {"key": "status", "type": "str"},
        "filters": {"key": "filters", "type": "[KeyValueFilter]"},
        "composition_type": {"key": "composition_type", "type": "str"},
        "created": {"key": "created", "type": "iso-8601"},
        "expires": {"key": "expires", "type": "iso-8601"},
        "retention_period": {"key": "retention_period", "type": "int"},
        "size": {"key": "size", "type": "int"},
        "items_count": {"key": "items_count", "type": "int"},
        "tags": {"key": "tags", "type": "{str}"},
        "etag": {"key": "etag", "type": "str"},
    }

    def __init__(
        self,
        *,
        filters: List["_models.KeyValueFilter"],
        composition_type: Optional[Union[str, "_models.CompositionType"]] = None,
        retention_period: Optional[int] = None,
        tags: Optional[Dict[str, str]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword filters: A list of filters used to filter the key-values included in the snapshot.
         Required.
        :paramtype filters: list[~azure.appconfiguration.models.KeyValueFilter]
        :keyword composition_type: The composition type describes how the key-values within the
         snapshot are composed. The 'key' composition type ensures there are no two key-values
         containing the same key. The 'key_label' composition type ensures there are no two key-values
         containing the same key and label. Known values are: "key" and "key_label".
        :paramtype composition_type: str or ~azure.appconfiguration.models.CompositionType
        :keyword retention_period: The amount of time, in seconds, that a snapshot will remain in the
         archived state before expiring. This property is only writable during the creation of a
         snapshot. If not specified, the default lifetime of key-value revisions will be used.
        :paramtype retention_period: int
        :keyword tags: The tags of the snapshot.
        :paramtype tags: dict[str, str]
        """
        super().__init__(**kwargs)
        self.name = None
        self.status = None
        self.filters = filters
        self.composition_type = composition_type
        self.created = None
        self.expires = None
        self.retention_period = retention_period
        self.size = None
        self.items_count = None
        self.tags = tags
        self.etag = None


class SnapshotListResult(_serialization.Model):
    """The result of a snapshot list request.

    :ivar items: The collection value.
    :vartype items: list[~azure.appconfiguration.models.Snapshot]
    :ivar next_link: The URI that can be used to request the next set of paged results.
    :vartype next_link: str
    """

    _attribute_map = {
        "items": {"key": "items", "type": "[Snapshot]"},
        "next_link": {"key": "@nextLink", "type": "str"},
    }

    def __init__(
        self, *, items: Optional[List["_models.Snapshot"]] = None, next_link: Optional[str] = None, **kwargs: Any
    ) -> None:
        """
        :keyword items: The collection value.
        :paramtype items: list[~azure.appconfiguration.models.Snapshot]
        :keyword next_link: The URI that can be used to request the next set of paged results.
        :paramtype next_link: str
        """
        super().__init__(**kwargs)
        self.items = items
        self.next_link = next_link


class SnapshotUpdateParameters(_serialization.Model):
    """Parameters used to update a snapshot.

    :ivar status: The desired status of the snapshot. Known values are: "provisioning", "ready",
     "archived", and "failed".
    :vartype status: str or ~azure.appconfiguration.models.SnapshotStatus
    """

    _attribute_map = {
        "status": {"key": "status", "type": "str"},
    }

    def __init__(self, *, status: Optional[Union[str, "_models.SnapshotStatus"]] = None, **kwargs: Any) -> None:
        """
        :keyword status: The desired status of the snapshot. Known values are: "provisioning", "ready",
         "archived", and "failed".
        :paramtype status: str or ~azure.appconfiguration.models.SnapshotStatus
        """
        super().__init__(**kwargs)
        self.status = status



# --------------------------------------------------------------------------
#
# Copyright (c) Microsoft Corporation. All rights reserved.
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the ""Software""), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
# --------------------------------------------------------------------------
from typing import Any, Dict, Optional
from asyncio import Lock
from azure.core.pipeline import PipelineRequest, PipelineResponse
from azure.core.pipeline.policies import SansIOHTTPPolicy

from .._sync_token import SyncToken


class AsyncSyncTokenPolicy(SansIOHTTPPolicy):
    """A simple policy that enable the given callback with the response.

    :keyword callback raw_response_hook: Callback function. Will be invoked on response.
    """

    def __init__(self, **kwargs: Any) -> None:  # pylint: disable=unused-argument
        self._sync_token_header = "Sync-Token"
        self._sync_tokens: Dict[str, Any] = {}
        self._lock = Lock()

    async def on_request(self, request: PipelineRequest) -> None:  # pylint: disable=invalid-overridden-method
        """This is executed before sending the request to the next policy.

        :param request: The PipelineRequest object.
        :type request: ~azure.core.pipeline.PipelineRequest
        """
        async with self._lock:
            sync_token_header = ",".join(str(x) for x in self._sync_tokens.values())
            if sync_token_header:
                request.http_request.headers.update({self._sync_token_header: sync_token_header})

    async def on_response(  # pylint: disable=invalid-overridden-method
        self, request: PipelineRequest, response: PipelineResponse
    ) -> None:
        """This is executed after the request comes back from the policy.

        :param request: The PipelineRequest object.
        :type request: ~azure.core.pipeline.PipelineRequest
        :param response: The PipelineResponse object.
        :type response: ~azure.core.pipeline.PipelineResponse
        """
        sync_token_header = response.http_response.headers.get(self._sync_token_header)
        if not sync_token_header:
            return
        sync_token_strings = sync_token_header.split(",")
        if not sync_token_strings:
            return
        for sync_token_string in sync_token_strings:
            sync_token = SyncToken.from_sync_token_string(sync_token_string)
            await self._update_sync_token(sync_token)

    async def add_token(self, full_raw_tokens: str) -> None:
        raw_tokens = full_raw_tokens.split(",")
        for raw_token in raw_tokens:
            sync_token = SyncToken.from_sync_token_string(raw_token)
            await self._update_sync_token(sync_token)

    async def _update_sync_token(self, sync_token: Optional[SyncToken]) -> None:
        if not sync_token:
            return
        async with self._lock:
            existing_token = self._sync_tokens.get(sync_token.token_id, None)
            if not existing_token:
                self._sync_tokens[sync_token.token_id] = sync_token
                return
            if existing_token.sequence_number < sync_token.sequence_number:
                self._sync_tokens[sync_token.token_id] = sync_token



# coding: utf-8
# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from preparers import AppConfigPreparer, trim_kwargs_from_test_function


def app_config_decorator_async(func, **kwargs):
    @AppConfigPreparer()
    async def wrapper(*args, **kwargs):
        appconfiguration_connection_string = kwargs.pop("appconfiguration_connection_string")
        kwargs["appconfiguration_connection_string"] = appconfiguration_connection_string

        trimmed_kwargs = {k: v for k, v in kwargs.items()}
        trim_kwargs_from_test_function(func, trimmed_kwargs)

        await func(*args, **trimmed_kwargs)

    return wrapper


def app_config_aad_decorator_async(func, **kwargs):
    @AppConfigPreparer()
    async def wrapper(*args, **kwargs):
        appconfiguration_endpoint_string = kwargs.pop("appconfiguration_endpoint_string")
        kwargs["appconfiguration_endpoint_string"] = appconfiguration_endpoint_string

        trimmed_kwargs = {k: v for k, v in kwargs.items()}
        trim_kwargs_from_test_function(func, trimmed_kwargs)

        await func(*args, **trimmed_kwargs)

    return wrapper



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: hello_world_sample.py

DESCRIPTION:
    This sample demos set/get/delete operations for app configuration

USAGE: python hello_world_sample.py

    Set the environment variables with your own values before running the sample:
    1) APPCONFIGURATION_CONNECTION_STRING: Connection String used to access the Azure App Configuration.
"""
from azure.appconfiguration import ConfigurationSetting
from util import print_configuration_setting


def main():
    # [START create_app_config_client]
    import os
    from azure.appconfiguration import AzureAppConfigurationClient

    CONNECTION_STRING = os.environ["APPCONFIGURATION_CONNECTION_STRING"]

    # Create app config client
    client = AzureAppConfigurationClient.from_connection_string(CONNECTION_STRING)
    # [END create_app_config_client]

    print("Set new configuration setting")
    config_setting = ConfigurationSetting(
        key="MyKey", value="my value", content_type="my content type", tags={"my tag": "my tag value"}
    )
    returned_config_setting = client.set_configuration_setting(config_setting)
    print("New configuration setting:")
    print_configuration_setting(returned_config_setting)
    print("")

    print("Get configuration setting")
    fetched_config_setting = client.get_configuration_setting(key="MyKey")
    print("Fetched configuration setting:")
    print_configuration_setting(fetched_config_setting)
    print("")

    print("Delete configuration setting")
    client.delete_configuration_setting(key="MyKey")


if __name__ == "__main__":
    main()



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: list_revision_sample.py

DESCRIPTION:
    This sample demos list revision operations for app configuration

USAGE: python list_revision_sample.py

    Set the environment variables with your own values before running the sample:
    1) APPCONFIGURATION_CONNECTION_STRING: Connection String used to access the Azure App Configuration.
"""
import os
from azure.appconfiguration import AzureAppConfigurationClient, ConfigurationSetting
from util import print_configuration_setting


def main():
    CONNECTION_STRING = os.environ["APPCONFIGURATION_CONNECTION_STRING"]

    # Create app config client
    client = AzureAppConfigurationClient.from_connection_string(CONNECTION_STRING)

    config_setting = ConfigurationSetting(
        key="MyKey", value="my value", content_type="my content type", tags={"my tag": "my tag value"}
    )
    returned_config_setting = client.set_configuration_setting(config_setting)

    returned_config_setting.value = "new value"
    returned_config_setting.content_type = "new content type"
    client.set_configuration_setting(config_setting)

    items = client.list_revisions(key_filter="MyKey")
    for item in items:
        print_configuration_setting(item)
        print("")

    client.delete_configuration_setting(
        key="MyKey",
    )


if __name__ == "__main__":
    main()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from azure.core import MatchConditions
from azure.core.exceptions import (
    ResourceModifiedError,
    ResourceNotFoundError,
    ResourceExistsError,
    AzureError,
)
from azure.appconfiguration import (
    ResourceReadOnlyError,
    ConfigurationSetting,
    SecretReferenceConfigurationSetting,
    FeatureFlagConfigurationSetting,
    FILTER_PERCENTAGE,
    FILTER_TARGETING,
    FILTER_TIME_WINDOW,
)
from asynctestcase import AsyncAppConfigTestCase
from consts import (
    KEY,
    LABEL,
    TEST_VALUE,
    TEST_CONTENT_TYPE,
    LABEL_RESERVED_CHARS,
    PAGE_SIZE,
    KEY_UUID,
)
from async_preparers import app_config_aad_decorator_async
from devtools_testutils import set_custom_default_matcher
from devtools_testutils.aio import recorded_by_proxy_async
import pytest
import copy
import datetime
import json
import re
import copy


class TestAppConfigurationClientAADAsync(AsyncAppConfigTestCase):
    # method: add_configuration_setting
    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_add_configuration_setting(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            test_config_setting = ConfigurationSetting(
                key=KEY + "_ADD",
                label=LABEL,
                value=TEST_VALUE,
                content_type=TEST_CONTENT_TYPE,
                tags={"tag1": "tag1", "tag2": "tag2"},
            )
            created_kv = await client.add_configuration_setting(test_config_setting)
            assert (
                created_kv.label == test_config_setting.label
                and created_kv.value == test_config_setting.value
                and created_kv.content_type == test_config_setting.content_type
                and created_kv.tags == test_config_setting.tags
                and created_kv.etag != None
                and created_kv.etag != test_config_setting.etag
                and created_kv.last_modified != None
                and created_kv.read_only == False
            )

            # test add existing configuration setting
            with pytest.raises(ResourceExistsError):
                await client.add_configuration_setting(
                    ConfigurationSetting(
                        key=test_config_setting.key,
                        label=test_config_setting.label,
                    )
                )
            await client.delete_configuration_setting(key=created_kv.key, label=created_kv.label)

    # method: set_configuration_setting
    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_set_configuration_setting(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            to_set_kv = self.create_config_setting()
            to_set_kv.value = to_set_kv.value + "a"
            to_set_kv.tags = {"a": "b", "c": "d"}
            set_kv = await client.set_configuration_setting(to_set_kv)
            assert (
                to_set_kv.key == set_kv.key
                and to_set_kv.label == to_set_kv.label
                and to_set_kv.value == set_kv.value
                and to_set_kv.content_type == set_kv.content_type
                and to_set_kv.tags == set_kv.tags
                and to_set_kv.etag != set_kv.etag
            )
            await client.delete_configuration_setting(key=to_set_kv.key, label=to_set_kv.label)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_set_configuration1_setting_with_wrong_etag(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            to_set_kv = self.create_config_setting()
            to_set_kv.value = to_set_kv.value + "a"
            to_set_kv.tags = {"a": "b", "c": "d"}
            to_set_kv.etag = "wrong etag"
            with pytest.raises(ResourceModifiedError):
                await client.set_configuration_setting(to_set_kv, match_condition=MatchConditions.IfNotModified)

    # method: get_configuration_setting
    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_get_configuration_setting_no_label(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            compare_kv = self.create_config_setting_no_label()
            await self.add_for_test(client, compare_kv)
            fetched_kv = await client.get_configuration_setting(compare_kv.key)
            assert (
                fetched_kv.key == compare_kv.key
                and fetched_kv.value == compare_kv.value
                and fetched_kv.content_type == compare_kv.content_type
                and fetched_kv.tags == compare_kv.tags
            )
            assert fetched_kv.label is None
            await client.delete_configuration_setting(key=compare_kv.key, label=compare_kv.label)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_get_configuration_setting(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            compare_kv = self.create_config_setting()
            await self.add_for_test(client, compare_kv)
            fetched_kv = await client.get_configuration_setting(compare_kv.key, compare_kv.label)
            assert (
                fetched_kv.key == compare_kv.key
                and fetched_kv.value == compare_kv.value
                and fetched_kv.content_type == compare_kv.content_type
                and fetched_kv.tags == compare_kv.tags
                and fetched_kv.label == compare_kv.label
            )
            assert fetched_kv.label is not None
            await client.delete_configuration_setting(key=compare_kv.key, label=compare_kv.label)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_get_non_existing_configuration_setting(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            compare_kv = self.create_config_setting()
            with pytest.raises(ResourceNotFoundError):
                await client.get_configuration_setting(compare_kv.key, compare_kv.label + "a")

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_get_configuration_setting_with_etag(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            compare_kv = self.create_config_setting()
            await self.add_for_test(client, compare_kv)
            compare_kv = await client.get_configuration_setting(compare_kv.key, compare_kv.label)

            # test get with wrong etag
            with pytest.raises(ResourceModifiedError):
                await client.get_configuration_setting(
                    compare_kv.key, compare_kv.label, etag="wrong etag", match_condition=MatchConditions.IfNotModified
                )
            # test get with correct etag
            with pytest.raises(ResourceNotFoundError):
                await client.get_configuration_setting(compare_kv.key, etag=compare_kv.etag)
            await client.get_configuration_setting(compare_kv.key, compare_kv.label, etag=compare_kv.etag)

            await client.delete_configuration_setting(key=compare_kv.key, label=compare_kv.label)

    # method: delete_configuration_setting
    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_delete_configuration_setting_with_key_no_label(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            to_delete_kv = self.create_config_setting_no_label()
            await self.add_for_test(client, to_delete_kv)
            deleted_kv = await client.delete_configuration_setting(key=to_delete_kv.key, label=to_delete_kv.label)
            assert deleted_kv is not None
            with pytest.raises(ResourceNotFoundError):
                await client.get_configuration_setting(to_delete_kv.key)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_delete_configuration_setting_with_key_label(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            to_delete_kv = self.create_config_setting()
            await self.add_for_test(client, to_delete_kv)
            deleted_kv = await client.delete_configuration_setting(key=to_delete_kv.key, label=to_delete_kv.label)
            assert deleted_kv is not None
            with pytest.raises(ResourceNotFoundError):
                await client.get_configuration_setting(to_delete_kv.key, label=to_delete_kv.label)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_delete_not_existing_configuration_setting(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            deleted_kv = await client.delete_configuration_setting("not_exist_" + KEY)
            assert deleted_kv is None

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_delete_configuration_setting_with_etag(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            to_delete_kv = self.create_config_setting_no_label()
            await self.add_for_test(client, to_delete_kv)
            to_delete_kv = await client.get_configuration_setting(to_delete_kv.key, to_delete_kv.label)

            # test delete with wrong etag
            with pytest.raises(ResourceModifiedError):
                await client.delete_configuration_setting(
                    to_delete_kv.key, etag="wrong etag", match_condition=MatchConditions.IfNotModified
                )
            # test delete with correct etag
            deleted_kv = await client.delete_configuration_setting(to_delete_kv.key, etag=to_delete_kv.etag)
            assert deleted_kv is not None
            with pytest.raises(ResourceNotFoundError):
                await client.get_configuration_setting(to_delete_kv.key)

    # method: list_configuration_settings
    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_configuration_settings_key_label(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        items = await self.convert_to_list(self.client.list_configuration_settings(label_filter=LABEL, key_filter=KEY))
        assert len(items) == 1
        assert all(x.key == KEY and x.label == LABEL for x in items)
        await self.tear_down()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_configuration_settings_only_label(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        items = await self.convert_to_list(self.client.list_configuration_settings(label_filter=LABEL))
        assert len(items) == 1
        assert all(x.label == LABEL for x in items)
        await self.tear_down()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_configuration_settings_only_key(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        items = await self.convert_to_list(self.client.list_configuration_settings(key_filter=KEY))
        assert len(items) == 2
        assert all(x.key == KEY for x in items)
        await self.tear_down()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_configuration_settings_fields(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        items = await self.convert_to_list(
            self.client.list_configuration_settings(key_filter="*", label_filter=LABEL, fields=["key", "content_type"])
        )
        assert len(items) == 1
        assert all(x.key and not x.label and x.content_type for x in items)
        await self.tear_down()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_configuration_settings_reserved_chars(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            reserved_char_kv = ConfigurationSetting(key=KEY, label=LABEL_RESERVED_CHARS, value=TEST_VALUE)
            reserved_char_kv = await client.add_configuration_setting(reserved_char_kv)
            escaped_label = re.sub(r"((?!^)\*(?!$)|\\|,)", r"\\\1", LABEL_RESERVED_CHARS)
            items = await self.convert_to_list(client.list_configuration_settings(label_filter=escaped_label))
            assert len(items) == 1
            assert all(x.label == LABEL_RESERVED_CHARS for x in items)
            await client.delete_configuration_setting(reserved_char_kv.key)

    # NOTE: Label filter does not support wildcard at beginning on filters. https://docs.microsoft.com/en-us/azure/azure-app-configuration/rest-api-key-value#supported-filters
    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_configuration_settings_contains(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        items = await self.convert_to_list(self.client.list_configuration_settings(label_filter=LABEL + "*"))
        assert len(items) == 1
        assert all(x.label == LABEL for x in items)
        await self.tear_down()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_configuration_settings_correct_etag(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            to_list_kv = self.create_config_setting()
            await self.add_for_test(client, to_list_kv)
            to_list_kv = await client.get_configuration_setting(to_list_kv.key, to_list_kv.label)
            custom_headers = {"If-Match": to_list_kv.etag or ""}
            items = await self.convert_to_list(
                client.list_configuration_settings(
                    key_filter=to_list_kv.key, label_filter=to_list_kv.label, headers=custom_headers
                )
            )
            assert len(items) == 1
            assert all(x.key == to_list_kv.key and x.label == to_list_kv.label for x in items)
            await client.delete_configuration_setting(to_list_kv.key)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_configuration_settings_multi_pages(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            # create PAGE_SIZE+1 configuration settings to have at least two pages
            try:
                [
                    await client.add_configuration_setting(
                        ConfigurationSetting(
                            key="multi_" + str(i) + KEY_UUID,
                            label="multi_label_" + str(i),
                            value="multi value",
                        )
                    )
                    for i in range(PAGE_SIZE + 1)
                ]
            except ResourceExistsError:
                pass
            items = await self.convert_to_list(client.list_configuration_settings(key_filter="multi_*"))
            assert len(items) > PAGE_SIZE

            # Remove the configuration settings
            try:
                [
                    await client.delete_configuration_setting(
                        key="multi_" + str(i) + KEY_UUID, label="multi_label_" + str(i)
                    )
                    for i in range(PAGE_SIZE + 1)
                ]
            except AzureError:
                pass

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_configuration_settings_null_label(self, appconfiguration_endpoint_string):
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        items = await self.convert_to_list(self.client.list_configuration_settings(label_filter="\0"))
        assert len(items) > 0
        await self.tear_down()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_configuration_settings_only_accepttime(self, appconfiguration_endpoint_string, **kwargs):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        recorded_variables = kwargs.pop("variables", {})
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        exclude_today = await self.convert_to_list(
            self.client.list_configuration_settings(
                accept_datetime=recorded_variables.setdefault(
                    "datetime", str(datetime.datetime.today() + datetime.timedelta(days=-1))
                )
            )
        )
        all_inclusive = await self.convert_to_list(self.client.list_configuration_settings())
        assert len(all_inclusive) > len(exclude_today)
        await self.tear_down()
        return recorded_variables

    # method: list_revisions
    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_revisions_key_label(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        to_list = self.create_config_setting()
        items = await self.convert_to_list(
            self.client.list_revisions(label_filter=to_list.label, key_filter=to_list.key)
        )
        assert len(items) >= 2
        assert all(x.key == to_list.key and x.label == to_list.label for x in items)
        await self.tear_down()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_revisions_only_label(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        items = await self.convert_to_list(self.client.list_revisions(label_filter=LABEL))
        assert len(items) >= 1
        assert all(x.label == LABEL for x in items)
        await self.tear_down()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_revisions_key_no_label(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        items = await self.convert_to_list(self.client.list_revisions(key_filter=KEY))
        assert len(items) >= 1
        assert all(x.key == KEY for x in items)
        await self.tear_down()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_revisions_fields(self, appconfiguration_endpoint_string):
        # response header <x-ms-content-sha256> and <x-ms-date> are missing in python38.
        set_custom_default_matcher(compare_bodies=False, excluded_headers="x-ms-content-sha256,x-ms-date")
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        items = await self.convert_to_list(
            self.client.list_revisions(key_filter="*", label_filter=LABEL, fields=["key", "content_type"])
        )
        assert all(x.key and not x.label and x.content_type and not x.tags and not x.etag for x in items)
        await self.tear_down()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_list_revisions_correct_etag(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            to_list_kv = self.create_config_setting()
            await self.add_for_test(client, to_list_kv)
            to_list_kv = await client.get_configuration_setting(to_list_kv.key, to_list_kv.label)
            custom_headers = {"If-Match": to_list_kv.etag or ""}
            items = await self.convert_to_list(
                client.list_revisions(key_filter=to_list_kv.key, label_filter=to_list_kv.label, headers=custom_headers)
            )
            assert len(items) >= 1
            assert all(x.key == to_list_kv.key and x.label == to_list_kv.label for x in items)

            await client.delete_configuration_setting(to_list_kv.key)

    # method: set_read_only
    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_set_read_only(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            to_set_kv = self.create_config_setting()
            await self.add_for_test(client, to_set_kv)
            to_set_kv = await client.get_configuration_setting(to_set_kv.key, to_set_kv.label)

            read_only_kv = await client.set_read_only(to_set_kv)
            assert read_only_kv.read_only
            with pytest.raises(ResourceReadOnlyError):
                await client.set_configuration_setting(read_only_kv)
            with pytest.raises(ResourceReadOnlyError):
                await client.delete_configuration_setting(read_only_kv.key, read_only_kv.label)

            writable_kv = await client.set_read_only(read_only_kv, False)
            assert not writable_kv.read_only
            await client.set_configuration_setting(writable_kv)
            await client.delete_configuration_setting(to_set_kv.key)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_set_read_only_with_wrong_etag(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            to_set_kv = self.create_config_setting()
            await self.add_for_test(client, to_set_kv)
            to_set_kv = await client.get_configuration_setting(to_set_kv.key, to_set_kv.label)

            to_set_kv.etag = "wrong etag"
            with pytest.raises(ResourceModifiedError):
                await client.set_read_only(to_set_kv, False, match_condition=MatchConditions.IfNotModified)

            await client.delete_configuration_setting(to_set_kv)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_sync_tokens_with_configuration_setting(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            sync_tokens = copy.deepcopy(client._sync_token_policy._sync_tokens)
            sync_token_header = self._order_dict(sync_tokens)
            sync_token_header = ",".join(str(x) for x in sync_token_header.values())

            new = ConfigurationSetting(
                key="KEY1",
                label=None,
                value="TEST_VALUE1",
                content_type=TEST_CONTENT_TYPE,
                tags={"tag1": "tag1", "tag2": "tag2"},
            )

            await client.set_configuration_setting(new)
            sync_tokens2 = copy.deepcopy(client._sync_token_policy._sync_tokens)
            sync_token_header2 = self._order_dict(sync_tokens2)
            sync_token_header2 = ",".join(str(x) for x in sync_token_header2.values())
            assert sync_token_header != sync_token_header2

            new = ConfigurationSetting(
                key="KEY2",
                label=None,
                value="TEST_VALUE2",
                content_type=TEST_CONTENT_TYPE,
                tags={"tag1": "tag1", "tag2": "tag2"},
            )

            await client.set_configuration_setting(new)
            sync_tokens3 = copy.deepcopy(client._sync_token_policy._sync_tokens)
            sync_token_header3 = self._order_dict(sync_tokens3)
            sync_token_header3 = ",".join(str(x) for x in sync_token_header3.values())
            assert sync_token_header2 != sync_token_header3

            await client.delete_configuration_setting(new.key)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_sync_tokens_with_feature_flag_configuration_setting(self, appconfiguration_endpoint_string):
        await self.set_up(appconfiguration_endpoint_string, is_aad=True)
        new = FeatureFlagConfigurationSetting(
            "custom",
            enabled=True,
            filters=[
                {
                    "name": "Microsoft.Percentage",
                    "parameters": {
                        "Value": 10,
                        "User": "user1",
                    },
                }
            ],
        )

        sync_tokens = copy.deepcopy(self.client._sync_token_policy._sync_tokens)
        keys = list(sync_tokens.keys())
        seq_num = sync_tokens[keys[0]].sequence_number
        await self.client.set_configuration_setting(new)

        new = FeatureFlagConfigurationSetting(
            "time_window",
            enabled=True,
            filters=[
                {
                    "name": FILTER_TIME_WINDOW,
                    "parameters": {"Start": "Wed, 10 Mar 2021 05:00:00 GMT", "End": "Fri, 02 Apr 2021 04:00:00 GMT"},
                },
            ],
        )

        await self.client.set_configuration_setting(new)
        sync_tokens2 = copy.deepcopy(self.client._sync_token_policy._sync_tokens)
        keys = list(sync_tokens2.keys())
        seq_num2 = sync_tokens2[keys[0]].sequence_number

        new = FeatureFlagConfigurationSetting(
            "newflag",
            enabled=True,
            filters=[
                {
                    "name": FILTER_TARGETING,
                    "parameters": {
                        "Audience": {"Users": ["abc", "def"], "Groups": ["ghi", "jkl"], "DefaultRolloutPercentage": 75}
                    },
                },
            ],
        )

        await self.client.set_configuration_setting(new)
        sync_tokens3 = copy.deepcopy(self.client._sync_token_policy._sync_tokens)
        keys = list(sync_tokens3.keys())
        seq_num3 = sync_tokens3[keys[0]].sequence_number

        assert seq_num < seq_num2
        assert seq_num2 < seq_num3

        await self.client.delete_configuration_setting(new.key)
        await self.client.close()

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_config_setting_feature_flag(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            feature_flag = FeatureFlagConfigurationSetting("test_feature", enabled=True)

            set_flag = await client.set_configuration_setting(feature_flag)
            self._assert_same_keys(feature_flag, set_flag)
            set_flag_value = json.loads(set_flag.value)
            assert set_flag_value["id"] == "test_feature"
            assert set_flag_value["enabled"] == True
            assert set_flag_value["conditions"] != None

            set_flag.enabled = not set_flag.enabled
            changed_flag = await client.set_configuration_setting(set_flag)
            assert changed_flag.enabled == False
            temp = json.loads(changed_flag.value)
            assert temp["id"] == set_flag_value["id"]
            assert temp["enabled"] == False
            assert temp["conditions"] == set_flag_value["conditions"]

            c = json.loads(changed_flag.value)
            c["enabled"] = True
            changed_flag.value = json.dumps(c)
            assert changed_flag.enabled == True
            temp = json.loads(changed_flag.value)
            assert temp["id"] == set_flag_value["id"]
            assert temp["enabled"] == True
            assert temp["conditions"] == set_flag_value["conditions"]

            changed_flag.value = json.dumps({})
            assert changed_flag.enabled == False
            temp = json.loads(changed_flag.value)
            assert temp["id"] == set_flag_value["id"]
            assert temp["enabled"] == False
            assert temp["conditions"] != None
            assert temp["conditions"]["client_filters"] == None

            set_flag.value = "bad_value"
            assert set_flag.enabled == False
            assert set_flag.filters == None
            assert set_flag.value == "bad_value"

            await client.delete_configuration_setting(changed_flag.key)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_config_setting_secret_reference(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            secret_reference = SecretReferenceConfigurationSetting(
                "ConnectionString", "https://test-test.vault.azure.net/secrets/connectionString"
            )
            set_flag = await client.set_configuration_setting(secret_reference)
            self._assert_same_keys(secret_reference, set_flag)

            set_flag.secret_id = "https://test-test.vault.azure.net/new_secrets/connectionString"
            updated_flag = await client.set_configuration_setting(set_flag)
            self._assert_same_keys(set_flag, updated_flag)

            assert isinstance(updated_flag, SecretReferenceConfigurationSetting)
            new_uri = "https://aka.ms/azsdk"
            new_uri2 = "https://aka.ms/azsdk/python"
            updated_flag.secret_id = new_uri
            temp = json.loads(updated_flag.value)
            assert temp["uri"] == new_uri

            updated_flag.value = json.dumps({"uri": new_uri2})
            assert updated_flag.secret_id == new_uri2

            set_flag.value = "bad_value"
            assert set_flag.secret_id == None

            await client.delete_configuration_setting(secret_reference.key)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_feature_filter_targeting(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            new = FeatureFlagConfigurationSetting(
                "newflag",
                enabled=True,
                filters=[
                    {
                        "name": FILTER_TARGETING,
                        "parameters": {
                            "Audience": {
                                "Users": ["abc", "def"],
                                "Groups": ["ghi", "jkl"],
                                "DefaultRolloutPercentage": 75,
                            }
                        },
                    }
                ],
            )

            sent_config = await client.set_configuration_setting(new)
            self._assert_same_keys(sent_config, new)

            assert isinstance(sent_config.filters[0], dict)
            assert len(sent_config.filters) == 1

            sent_config.filters[0]["parameters"]["Audience"]["DefaultRolloutPercentage"] = 80
            updated_sent_config = await client.set_configuration_setting(sent_config)
            self._assert_same_keys(sent_config, updated_sent_config)

            updated_sent_config.filters.append(
                {
                    "name": FILTER_TARGETING,
                    "parameters": {
                        "Audience": {
                            "Users": ["abcd", "defg"],  # cspell:disable-line
                            "Groups": ["ghij", "jklm"],  # cspell:disable-line
                            "DefaultRolloutPercentage": 50,
                        }
                    },
                }
            )
            updated_sent_config.filters.append(
                {
                    "name": FILTER_TARGETING,
                    "parameters": {
                        "Audience": {
                            "Users": ["abcde", "defgh"],  # cspell:disable-line
                            "Groups": ["ghijk", "jklmn"],  # cspell:disable-line
                            "DefaultRolloutPercentage": 100,
                        }
                    },
                }
            )
            sent_config = await client.set_configuration_setting(updated_sent_config)
            self._assert_same_keys(sent_config, updated_sent_config)
            assert len(sent_config.filters) == 3

            await client.delete_configuration_setting(updated_sent_config.key)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_feature_filter_time_window(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            new = FeatureFlagConfigurationSetting(
                "time_window",
                enabled=True,
                filters=[
                    {
                        "name": FILTER_TIME_WINDOW,
                        "parameters": {
                            "Start": "Wed, 10 Mar 2021 05:00:00 GMT",
                            "End": "Fri, 02 Apr 2021 04:00:00 GMT",
                        },
                    }
                ],
            )

            sent = await client.set_configuration_setting(new)
            self._assert_same_keys(sent, new)

            sent.filters[0]["parameters"]["Start"] = "Thurs, 11 Mar 2021 05:00:00 GMT"
            new_sent = await client.set_configuration_setting(sent)
            self._assert_same_keys(sent, new_sent)

            await client.delete_configuration_setting(new_sent.key)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_feature_filter_custom(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            new = FeatureFlagConfigurationSetting(
                "custom",
                enabled=True,
                filters=[{"name": FILTER_PERCENTAGE, "parameters": {"Value": 10, "User": "user1"}}],
            )

            sent = await client.set_configuration_setting(new)
            self._assert_same_keys(sent, new)

            sent.filters[0]["parameters"]["Value"] = 100
            new_sent = await client.set_configuration_setting(sent)
            self._assert_same_keys(sent, new_sent)

            await client.delete_configuration_setting(new_sent.key)

    @app_config_aad_decorator_async
    @recorded_by_proxy_async
    async def test_feature_filter_multiple(self, appconfiguration_endpoint_string):
        async with self.create_aad_client(appconfiguration_endpoint_string) as client:
            new = FeatureFlagConfigurationSetting(
                "custom",
                enabled=True,
                filters=[
                    {"name": FILTER_PERCENTAGE, "parameters": {"Value": 10}},
                    {
                        "name": FILTER_TIME_WINDOW,
                        "parameters": {
                            "Start": "Wed, 10 Mar 2021 05:00:00 GMT",
                            "End": "Fri, 02 Apr 2021 04:00:00 GMT",
                        },
                    },
                    {
                        "name": FILTER_TARGETING,
                        "parameters": {
                            "Audience": {
                                "Users": ["abcde", "defgh"],  # cspell:disable-line
                                "Groups": ["ghijk", "jklmn"],  # cspell:disable-line
                                "DefaultRolloutPercentage": 100,
                            }
                        },
                    },
                ],
            )

            sent = await client.set_configuration_setting(new)
            self._assert_same_keys(sent, new)

            sent.filters[0]["parameters"]["Value"] = 100
            sent.filters[1]["parameters"]["Start"] = "Wed, 10 Mar 2021 08:00:00 GMT"
            sent.filters[2]["parameters"]["Audience"]["DefaultRolloutPercentage"] = 100

            new_sent = await client.set_configuration_setting(sent)
            self._assert_same_keys(sent, new_sent)

            assert new_sent.filters[0]["parameters"]["Value"] == 100
            assert new_sent.filters[1]["parameters"]["Start"] == "Wed, 10 Mar 2021 08:00:00 GMT"
            assert new_sent.filters[2]["parameters"]["Audience"]["DefaultRolloutPercentage"] == 100

            await client.delete_configuration_setting(new_sent.key)
